//https://www.shadertoy.com/view/ddlyW2
precision highp float;

uniform vec3                iResolution;
uniform float               iTime;
uniform sampler2D           iChannel0;
uniform sampler2D			iChannel1;
varying vec2                texCoord;
uniform float               intensity;
vec3 hash32(vec2 p) {
	return fract(vec3(
	sin(dot(p, vec2(127.1, 311.7))),
	sin(dot(p, vec2(269.5, 183.3))),
	sin(dot(p, vec2(419.2, 371.9)))
	) * 43758.5453);
}

float getLuminance(vec3 color) {
	return dot(color.rgb, vec3(0.299, 0.587, 0.114));
}


vec3 grain(vec2 uv, vec3 col, float intensity, float threshold) {
	vec3 noise = hash32(uv + vec2(fract(iTime)));

	float crispNoise = step(getLuminance(noise), threshold);

	noise = mix (vec3(0.5), noise, intensity);

	vec3 screen = 1.0 - 2.0 * (1.0 - col) * (1.0 - noise);
	vec3 mult = 2.0 * col * noise;

	vec3 overlay = mix(screen, mult, step(col, vec3(0.5)));

	vec3 mapMidtones = mix(col, overlay, 1.0 - distance(col, vec3(0.5)));

	vec3 final = mapMidtones + vec3(crispNoise);

	return final;
}


void mainImage( out vec4 fragColor, in vec2 fragCoord )
{
	vec2 uv = fragCoord/iResolution.xy;

	vec3 col = texture2D(iChannel0, uv).rgb;
	col = grain(uv, col, intensity*2.0, 0.01);

	fragColor = vec4(col,1.0);
}
void main() {
	mainImage(gl_FragColor, texCoord * iResolution.xy);
}