precision highp float;

uniform vec3                iResolution;
uniform float               iTime;
uniform sampler2D           iChannel0;
uniform sampler2D			iChannel1;
varying vec2                texCoord;
uniform float               intensity;

// Licensed under the MIT license:
// https://opensource.org/licenses/MIT.

#define USE_COLOR_GRAIN

// From Dave Hoskins: https://www.shadertoy.com/view/4djSRW
float hash(vec3 p3){
	p3 = fract(p3 * 0.1031);
	p3 += dot(p3,p3.yzx + 19.19);
	return fract((p3.x + p3.y) * p3.z);
}

// From iq: https://www.shadertoy.com/view/4sfGzS
float noise(vec3 x){
	vec3 i = floor(x);
	vec3 f = fract(x);
	f = f*f*(3.0-2.0*f);
	return mix(mix(mix(hash(i+vec3(0, 0, 0)),
	hash(i+vec3(1, 0, 0)),f.x),
	mix(hash(i+vec3(0, 1, 0)),
	hash(i+vec3(1, 1, 0)),f.x),f.y),
	mix(mix(hash(i+vec3(0, 0, 1)),
	hash(i+vec3(1, 0, 1)),f.x),
	mix(hash(i+vec3(0, 1, 1)),
	hash(i+vec3(1, 1, 1)),f.x),f.y),f.z);
}

// Slightly high-passed continuous value-noise.
float grain_source(vec3 x, float strength, float pitch){
	float center = noise(x);
	float v1 = center - noise(vec3( 1, 0, 0)/pitch + x) + 0.5;
	float v2 = center - noise(vec3( 0, 1, 0)/pitch + x) + 0.5;
	float v3 = center - noise(vec3(-1, 0, 0)/pitch + x) + 0.5;
	float v4 = center - noise(vec3( 0,-1, 0)/pitch + x) + 0.5;

	float total = (v1 + v2 + v3 + v4) / 4.0;
	return mix(1.0, 0.5 + total, strength);
}

void mainImage(out vec4 fragColor, in vec2 fragCoord){
	vec2 uv = fragCoord / iResolution.xy;

	vec2 x = fragCoord;
	// Alternatively use iTime here instead and change the grain_rate
	// parameter to correspond to frames-per-second.
	float t = mix(0.0, float(0.0), step(1.0, uv.x));
	vec3 color = texture2D(iChannel0, uv).rgb;
	vec3 grain = vec3(0);

	const float grain_strength = 1.0;
	const float grain_rate = 1.0;
	// Range: [0.5, 1.0].
	const float grain_pitch = 1.0;

	#ifdef USE_COLOR_GRAIN
	float rg = grain_source(vec3(x, floor(grain_rate*(t))),     grain_strength, grain_pitch);
	float gg = grain_source(vec3(x, floor(grain_rate*(t+9.0))), grain_strength, grain_pitch);
	float bg = grain_source(vec3(x, floor(grain_rate*(t-9.0))), grain_strength, grain_pitch);

	// Consider using values outside the [0, 1] range as well
	// to introduce interesting color shifts to the source.
	const float color_level = 1.0;
	vec3 color_grain = vec3(rg, gg, bg);
	color_grain = mix(vec3(dot(color_grain, vec3(0.2126, 0.7152, 0.0722))), color_grain, color_level);
	grain = color_grain;
	#else
	const float neutral_grain_factor = sqrt(2.0);
	grain = vec3(grain_source(vec3(x, floor(grain_rate*t)), grain_strength/neutral_grain_factor, grain_pitch));
	#endif

	// Control whether to add or multiply or lift the source with the grain.
	// Multiply (0.0) should be more true to life, but adjust to taste.
	float grain_lift_ratio = intensity * 4.0;
	color = max(mix(color*grain, color+(grain-1.0), grain_lift_ratio), 0.0);

	// After this you would normally perform tone mapping,
	// apply the grain before that.
	fragColor.rgb = color;
}

void main() {
	mainImage(gl_FragColor, texCoord * iResolution.xy);
}